import { useMemo, useContext } from "react";
import { apiContext } from "../../../api/DeployApi";
import { Grid, Box, Stack, Typography } from "@mui/material";
import {
  KeyValueTable,
  useAPIMethod,
  AlertBannerList
} from "@ess-ics/ce-ui-common";
import { IOCStatus } from "../IOCTable/IOCStatus";
import AccessControl from "../../auth/AccessControl";

export function diff(o1, o2) {
  let diff = Object.keys(o2).reduce((diff, key) => {
    if (o1[key] === o2[key]) {
      return diff;
    }
    return {
      ...diff,
      [key]: o2[key]
    };
  }, {});
  return diff;
}

const defaultSubset = ({
  name,
  description,
  host,
  active,
  status,
  git,
  version
}) => ({ name, description, host, active, status, git, version });

export function IOCDetails({ ioc, getSubset = defaultSubset, buttons }) {
  ioc = { ...ioc };
  const subset = getSubset(ioc);
  const client = useContext(apiContext);

  const alertParams = useMemo(
    () => ({
      ioc_id: ioc.id
    }),
    [ioc.id]
  );

  const { value: alert } = useAPIMethod({
    fcn: client.apis.IOCs.fetchIocAlerts,
    params: alertParams
  });

  return (
    <>
      <Grid
        container
        spacing={1}
      >
        {alert && alert.alerts.length > 0 ? (
          <Box sx={{ mt: (theme) => theme.spacing(2), width: "100%" }}>
            <AlertBannerList alerts={alert.alerts} />
          </Box>
        ) : null}
        {buttons && (
          <Grid
            item
            xs={12}
          >
            <Box
              display="flex"
              flexDirection="row-reverse"
              p={2}
              m={1}
            >
              <AccessControl
                allowedRoles={[
                  "DeploymentToolAdmin",
                  "DeploymentToolIntegrator"
                ]}
                renderNoAccess={() => <> </>}
              >
                <>{buttons}</>
              </AccessControl>
            </Box>
          </Grid>
        )}
        <Grid
          item
          xs={12}
        >
          <Stack
            flexDirection="row"
            alignItems="center"
            justifyContent="flex-start"
          >
            <Box
              sx={{
                marginRight: (theme) => theme.spacing(4),
                marginLeft: (theme) => theme.spacing(6)
              }}
            >
              <IOCStatus
                id={ioc.id}
                hideAlerts
              />
            </Box>
            <Box sx={{ flex: 1 }}>
              <Typography
                noWrap
                variant="subtitle1"
                component="p"
                color="textPrimary"
              >
                {ioc.namingName ?? ioc.name}
              </Typography>
              <Typography
                noWrap
                component="p"
                variant="subtitle2"
              >
                {ioc.activeDeployment?.host?.fqdn || "---"}
              </Typography>
            </Box>
          </Stack>

          <KeyValueTable
            obj={subset}
            variant="overline"
            sx={{ border: 0 }}
            valueOptions={{ headerName: "" }}
          />
        </Grid>
      </Grid>
    </>
  );
}
